"""
tests use this Component
<e
    Name="PYHELLODEMO1"
    Code="demoImpl.HelloDemo"
    Type="IDL:alma/demo/HelloDemo:1.0"
    Container="aragornContainer"
    ImplLang="py"
/>
with this IDL
module demo
{
    // a very simple component
    interface HelloDemo : ACS::ACSComponent
    {
        string sayHello();
        string sayHelloWithParameters(in string inString, inout double inoutDouble, out long outInt);
    };
};
"""
import pytest
from pprint import pprint
from bcs import manager

def test_manager_is_available_WITH_ACS():
    assert manager.is_available()

def test_simple_client_WITH_ACS():
    assert manager.simple_client() is not None

def test_manager_component_infos_WITH_ACS():
    some_dict = manager.component_infos()
    assert some_dict

"""
def test_manager_container_infos_WITH_ACS():
    print()
    some_dict = manager.container_infos()
    pprint(some_dict)
def test_manager_client_infos_WITH_ACS():
    print()
    some_dict = manager.client_infos()
    pprint(some_dict)
"""

def test_manager_get_component_WITH_ACS():
    """get_component will instantiate the component and we will be its client."""
    assert not manager.component_infos()["PYHELLODEMO1"].clients

    hello_demo = manager.get_component("PYHELLODEMO1")
    assert hello_demo is not None
    hello_demo.sayHello()

    assert manager.component_infos()["PYHELLODEMO1"].clients

def test_manager_restart_component_WITH_ACS():
    # the component must be running to be restarted
    if not manager.component_infos()["PYHELLODEMO1"].clients:
        manager.get_component("PYHELLODEMO1")

    manager.restart_component("PYHELLODEMO1")

def test_manager_set_mortality_WITH_ACS():
    """mortality is created by adding the ACS manager to the list of clients of a component"""
    num_clients = len(manager.component_infos()["PYHELLODEMO1"].clients)

    if num_clients == 0:
        # the component must be running to be immortalized
        manager.get_component("PYHELLODEMO1")
        num_clients = len(manager.component_infos()["PYHELLODEMO1"].clients)

    manager.set_mortality("PYHELLODEMO1", True)
    num_clients_after = len(manager.component_infos()["PYHELLODEMO1"].clients)
    assert num_clients_after == num_clients + 1


def test_manager_release_component_WITH_ACS():

    num_clients = len(manager.component_infos()["PYHELLODEMO1"].clients)
    if num_clients == 0:
        # makes ne sense releasing a component, when it is not running
        manager.get_component("PYHELLODEMO1")
        num_clients = len(manager.component_infos()["PYHELLODEMO1"].clients)

    manager.release_component("PYHELLODEMO1")
    num_clients_after = len(manager.component_infos()["PYHELLODEMO1"].clients)
    assert num_clients_after == num_clients - 1

def test_manager_try_activate_component_WITH_ACS():
    """ these try_.. methods have a weakly defined interface
    but in the best case it should return a component, we can use
    """
    comp = manager.try_activate_component("PYHELLODEMO1")
    comp.sayHello()


def test_manager_try_release_immortal_by_name_WITH_ACS():

    num_clients = len(manager.component_infos()["PYHELLODEMO1"].clients)
    if num_clients == 0:
        # its no fun trying this with no running component
        manager.get_component("PYHELLODEMO1")
        # aaand let's make it immortal
        manager.set_mortality("PYHELLODEMO1", True)
        num_clients = len(manager.component_infos()["PYHELLODEMO1"].clients)

    manager.try_release_immortal_by_name("PYHELLODEMO1")
    num_clients_after = len(manager.component_infos()["PYHELLODEMO1"].clients)
    assert num_clients_after <= num_clients

    # doing it again should not help a lot
    manager.try_release_immortal_by_name("PYHELLODEMO1")
    num_clients_again = len(manager.component_infos()["PYHELLODEMO1"].clients)
    assert num_clients_again == num_clients_after


def test_manager_try_release_immortal_by_prefix_WITH_ACS():
    num_clients = len(manager.component_infos()["PYHELLODEMO1"].clients)
    if num_clients == 0:
        # its no fun trying this with no running component
        manager.get_component("PYHELLODEMO1")
        # aaand let's make it immortal
        manager.set_mortality("PYHELLODEMO1", True)
        num_clients = len(manager.component_infos()["PYHELLODEMO1"].clients)

    manager.try_release_immortal_by_prefix("PYHELL*")
    num_clients_after = len(manager.component_infos()["PYHELLODEMO1"].clients)
    assert num_clients_after <= num_clients



def test_manager_get_component_dynamic_WITH_ACS():
    NAME = "dynamic_pyhello"

    assert not manager.component_infos(name_matches=NAME)
    manager.get_component(
        name=NAME,
        idl_type="IDL:alma/demo/HelloDemo:1.0",
        comp_code="demoImpl.HelloDemo",
        container_name="aragornContainer",
        is_dynamic=True,
    )
    assert manager.component_infos(name_matches=NAME)
